

function send(user, data) {
    var xhr = new XMLHttpRequest;

    try {
    xhr.open("POST", "/user/"  + user + "/run", true);
    xhr.onreadystatechange = function() {
        if (xhr.readyState == 4) {
            if (xhr.status != 200) { // this needs to be 201
              log("xhr failed? [" + xhr.status + "] " + xhr.statusText);
            } else {
              log("resource created at " + xhr.getResponseHeader("Location"));
            }
        } else {
            log("xhr readyState is " + xhr.readyState);
        }
    };
    xhr.setRequestHeader("Content-Type", "application/json");
    xhr.setRequestHeader("Content-Length", data.length);

    xhr.send(JSON.stringify(data));

    } catch (e) {
      log("Failed to send to server " + e);
      return false;
    }
   return true;
}

var storage = (function() {
     try {
       return !!window.localStorage.getItem;
     } catch (e) {
       return false;
     }
})();



var watchId = 0;
var running = false;

var duration = 0;
var start_time = 0;
var intervalId = 0;

// wait for at least 2 consecutive GPS data before declaring the GPS ready
var MAX_ERRORS = 2;
var got_error = MAX_ERRORS;

var discarded = 0;
var locations = 0;

var activity = { activity: "http://dbpedia.org/resource/Running", events: []};

var dst_unit = 1;

function formatNumber(n) {
  return (n < 10)? "0" + n : n;
}  

function formatTime(time) {
 t = parseInt(time);
 hours = parseInt(t / 3600);
 minutes = parseInt(t / 60);
 seconds = t - ((hours * 3600) + (minutes * 60));
 return ((hours > 0)?  formatNumber(hours) + ":": "") + formatNumber(minutes) + ":" + formatNumber(seconds);
}




function coordinates_event(position) {
   var events = activity.events;
   events[events.length] =
    { t:position.timestamp, c: [position.coords.longitude, position.coords.latitude, position.coords.altitude] };
}

function pause_event() {
   var events = activity.events;
   events[events.length] = { t:Date.now(), s: [ "pause" ] };
}

function calculateDistance(lat1, lon1, lat2, lon2) {
 var R = 6371; // km
 var dLat = (lat2-lat1) * Math.PI / 180;
 var dLon = (lon2-lon1) * Math.PI / 180;
 var lat1 = lat1  * Math.PI / 180;
 var lat2 = lat2 * Math.PI / 180;
 var a = Math.sin(dLat/2) * Math.sin(dLat/2) +
        Math.sin(dLon/2) * Math.sin(dLon/2) * Math.cos(lat1) * Math.cos(lat2); 
 var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a)); 
 var d = R * c;
 return d;
}

function calculateDistanceCoordinates() {
  var events = activity.events;
  var result = 0;
  if (events.length > 0) {
    var i = 1;
    while (i < events.length) {
      var position1 = events[i-1].c;
      var position2 = events[i].c;
      result += calculateDistance(position1[1], position1[0], position2[1], position2[0]);
      i++;
    }
  }
  return result;
}

function calculateDurationCoordinates() {
  var events = activity.events;
  var result = 0;
  if (events.length > 0) {
    var i = 1;
    while (i < events.length) {
      var position1 = events[i-1].t;
      var position2 = events[i].t;
      result += (position2 - position1);
      i++;
    }
  }
  return result;
}

function repaint() {
  if (document.visibilityState == true && running) {
    document.getElementById("locations").textContent = String(locations);
    var d = calculateDistanceCoordinates() * dst_unit;
    var t = duration + (Date.now() - start_time);
    document.getElementById("distance").textContent = String(Math.round(d*100)/100);
    if (t != 0 && d != 0) {
      document.getElementById("speed").textContent = String(Math.round( (d / (t / 3600000)) * 100) / 100);
      document.getElementById("pace").textContent = String(Math.round( ((t / 60000)/d) * 100) / 100);
    }
    if (activity.events.length > 0) {
      document.getElementById("gps_text").style.display = "none";
      document.getElementById("gps_data").style.display = "inline";
    }
  }
}


function log(msg) {
    document.getElementById("log").textContent = msg;
}

function logGPS(position, startMsg) {
    log(
      startMsg + position.timestamp 
       + ": " + position.coords.latitude
       + ", " + position.coords.longitude
       + ", " + position.coords.altitude
       + ", " + position.coords.accuracy
       + ", " + position.coords.altitudeAccuracy
       + ", " + position.coords.heading
       + ", " + position.coords.speed);
}

function handleSuccess(position) {
  if (running) {
   logGPS(position, "GPS: ");

    // discard positions that have no altitude since they're likely inacurate
    if (position.coords.altitude != null) {
      coordinates_event(position);
      locations++;
      repaint();
    } else {
      document.getElementById("discarded").textContent = String(discarded++);
    }
   } else {


     if (position.coords.altitude == null) {
       got_error++;
       if (got_error > MAX_ERRORS) got_error = MAX_ERRORS;
           logGPS(position, "[ERROR] GPS: ");
     } else {
       got_error--;
       logGPS(position, "GPS: ");
     }
     if (got_error <= 0) {
       got_error = 0;
       document.getElementById("gps_text").textContent = "ready";       
       document.getElementById("waves").style.display = "block";       
     } else {
       document.getElementById("gps_text").textContent = "not ready";
       document.getElementById("waves").style.display = "none";       
     }
   }
}

function handleError(error) {
  var code = "unknown";
  switch(error.code) {
    case error.TIMEOUT:
      code = "timeout";
      break;
    case error.PERMISSION_DENIED:
      code = "permission denied";
      break;
    case error.POSITION_UNAVAILABLE:
      code = "position unavailable";
      break;
  }
  if (running) {
    document.getElementById("errors").textContent = String(++got_error);
    log("[ERROR] GPS: " + code); 
  } else {
    document.getElementById("gps_text").textContent = code; 
    // wait for MAX_ERRORS non-consecutive errors
    got_error = MAX_ERRORS;
  }
}

function timer() {
  if (document.visibilityState == true) {
    document.getElementById("realtime").textContent = 
      formatTime(Math.round((duration+(Date.now()-start_time))/1000));
  }
}

function start() {
  if (!running) {
   running = true;
   start_time = Date.now();
   intervalId = setInterval(timer, 1000);
   document.getElementById("stop").disabled = false;
   document.getElementById("start").textContent = "Pause";
  } else {
   running = false;
   clearInterval(intervalId);
   duration += (Date.now() - start_time);
   document.getElementById("start").textContent = "Restart";
   pause_event();
   document.getElementById("gps_text").style.display = "inline";
   document.getElementById("gps_data").style.display = "none";
  }
}

function stop() {
  navigator.geolocation.clearWatch(watchId);
  running = false;
  clearInterval(intervalId);
  got_error = 0;
  locations = 0;
  discarded = 0;
  duration  = 0;
  document.getElementById("start").textContent = "Start";
  document.getElementById("stop").disabled = true;
  log("");
  if (storage) {
    try {
      window.localStorage.setItem("track_gps_app", JSON.stringify(activity));
    } catch (e) {
      log("[ERROR] Can't store the value " + e.code);
    }
  }
  if (!send("teole",activity)) {      
      // do something!      
  }
  activity.events = [];
}

function init() {  
  if (!!document.visibilityState)
    document.addEventListener("visibilitychange", repaint, false);
  else
    document.visibilityState = true;
  
  // send("teole", "{ activity:\"http://dbpedia.org/resource/Running\", events: [ {t: 1314049555597, c: [-71.09068298339844,42.361671924591064,66] },{t: 1314049559622,s: \"pause\"}]}");
  // return false;

  // @@ in the future, make sure this is empty
   if (storage) {
    window.localStorage.setItem("track_gps_app", "");

    if (window.localStorage.getItem("track_gps_app_dst_unit") === "mile") {
      dst_unit = 0.621371192;    
    }
    setUnit();
   } else {
     log("[ERROR] no local storage");
   }
   if (!!navigator.geolocation) {
     watchId = navigator.geolocation.watchPosition(handleSuccess, handleError, {enableHighAccuracy:true, maximumAge:0, timeout:1000});
   } else {
     document.getElementById("gps_text").textContent = "not supported";
     document.getElementById("start").disabled = true;
   }

}

window.addEventListener("load", init, false);

function km() {
  if (storage) {
    window.localStorage.setItem("track_gps_app_dst_unit", "km");
  }
  dst_unit = 1;
  setUnit();
  repaint();
}

function mile() {
  if (storage) {
    window.localStorage.setItem("track_gps_app_dst_unit", "mile");
  }
  dst_unit = 0.621371192;    
  setUnit();
  repaint();
}

function setUnit() {  
  var text = (dst_unit == 0.621371192)? "mile" : "km";
  var nodes = document.getElementsByClassName("dst_unit");
  i = 0;
  do {
    nodes[i].textContent = text;
  } while (++i < nodes.length);
}
